@@ -1,14 +1,9 @@
+from typing import List
 from unittest import TestCase
 
 from aioquic.quic.configuration import SMALLEST_MAX_DATAGRAM_SIZE
 from aioquic.quic.crypto import CryptoPair
-from aioquic.quic.packet import (
-    PACKET_TYPE_HANDSHAKE,
-    PACKET_TYPE_INITIAL,
-    PACKET_TYPE_ONE_RTT,
-    QuicFrameType,
-    QuicProtocolVersion,
-)
+from aioquic.quic.packet import QuicFrameType, QuicPacketType, QuicProtocolVersion
 from aioquic.quic.packet_builder import (
     QuicPacketBuilder,
     QuicPacketBuilderStop,
@@ -38,12 +33,16 @@ def create_crypto():
     return crypto
 
 
+def datagram_sizes(datagrams: List[bytes]) -> List[int]:
+    return [len(x) for x in datagrams]
+
+
 class QuicPacketBuilderTest(TestCase):
     def test_long_header_empty(self):
         builder = create_builder()
         crypto = create_crypto()
 
-        builder.start_packet(PACKET_TYPE_INITIAL, crypto)
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
         self.assertEqual(builder.remaining_flight_space, 1156)
         self.assertTrue(builder.packet_is_empty)
 
@@ -55,25 +54,24 @@ class QuicPacketBuilderTest(TestCase):
         # check builder
         self.assertEqual(builder.packet_number, 0)
 
-    def test_long_header_padding(self):
+    def test_long_header_initial_client(self):
         builder = create_builder(is_client=True)
         crypto = create_crypto()
 
         # INITIAL, fully padded
-        builder.start_packet(PACKET_TYPE_INITIAL, crypto)
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
         self.assertEqual(builder.remaining_flight_space, 1156)
         buf = builder.start_frame(QuicFrameType.CRYPTO)
         buf.push_bytes(bytes(100))
         self.assertFalse(builder.packet_is_empty)
 
         # INITIAL, empty
-        builder.start_packet(PACKET_TYPE_INITIAL, crypto)
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
         self.assertTrue(builder.packet_is_empty)
 
         # check datagrams
         datagrams, packets = builder.flush()
-        self.assertEqual(len(datagrams), 1)
-        self.assertEqual(len(datagrams[0]), 1200)
+        self.assertEqual(datagram_sizes(datagrams), [1200])
         self.assertEqual(
             packets,
             [
@@ -83,8 +81,8 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=True,
                     packet_number=0,
-                    packet_type=PACKET_TYPE_INITIAL,
-                    sent_bytes=1200,
+                    packet_type=QuicPacketType.INITIAL,
+                    sent_bytes=145,
                 )
             ],
         )
@@ -97,28 +95,26 @@ class QuicPacketBuilderTest(TestCase):
         crypto = create_crypto()
 
         # INITIAL, full length
-        builder.start_packet(PACKET_TYPE_INITIAL, crypto)
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
         self.assertEqual(builder.remaining_flight_space, 1156)
         buf = builder.start_frame(QuicFrameType.CRYPTO)
         buf.push_bytes(bytes(builder.remaining_flight_space))
         self.assertFalse(builder.packet_is_empty)
 
-        # INITIAL
-        builder.start_packet(PACKET_TYPE_INITIAL, crypto)
+        # INITIAL, full length
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
         self.assertEqual(builder.remaining_flight_space, 1156)
         buf = builder.start_frame(QuicFrameType.CRYPTO)
         buf.push_bytes(bytes(100))
         self.assertFalse(builder.packet_is_empty)
 
         # INITIAL, empty
-        builder.start_packet(PACKET_TYPE_INITIAL, crypto)
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
         self.assertTrue(builder.packet_is_empty)
 
         # check datagrams
         datagrams, packets = builder.flush()
-        self.assertEqual(len(datagrams), 2)
-        self.assertEqual(len(datagrams[0]), 1200)
-        self.assertEqual(len(datagrams[1]), 1200)
+        self.assertEqual(datagram_sizes(datagrams), [1200, 1200])
         self.assertEqual(
             packets,
             [
@@ -128,7 +124,7 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=True,
                     packet_number=0,
-                    packet_type=PACKET_TYPE_INITIAL,
+                    packet_type=QuicPacketType.INITIAL,
                     sent_bytes=1200,
                 ),
                 QuicSentPacket(
@@ -137,8 +133,56 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=True,
                     packet_number=1,
-                    packet_type=PACKET_TYPE_INITIAL,
-                    sent_bytes=1200,
+                    packet_type=QuicPacketType.INITIAL,
+                    sent_bytes=145,
+                ),
+            ],
+        )
+
+        # check builder
+        self.assertEqual(builder.packet_number, 2)
+
+    def test_long_header_initial_client_zero_rtt(self):
+        builder = create_builder(is_client=True)
+        crypto = create_crypto()
+
+        # INITIAL
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
+        self.assertEqual(builder.remaining_flight_space, 1156)
+        buf = builder.start_frame(QuicFrameType.CRYPTO)
+        buf.push_bytes(bytes(613))
+        self.assertFalse(builder.packet_is_empty)
+
+        # 0-RTT
+        builder.start_packet(QuicPacketType.ZERO_RTT, crypto)
+        self.assertEqual(builder.remaining_flight_space, 499)
+        buf = builder.start_frame(QuicFrameType.STREAM_BASE)
+        buf.push_bytes(bytes(100))
+        self.assertFalse(builder.packet_is_empty)
+
+        # check datagrams
+        datagrams, packets = builder.flush()
+        self.assertEqual(datagram_sizes(datagrams), [1200])
+        self.assertEqual(
+            packets,
+            [
+                QuicSentPacket(
+                    epoch=Epoch.INITIAL,
+                    in_flight=True,
+                    is_ack_eliciting=True,
+                    is_crypto_packet=True,
+                    packet_number=0,
+                    packet_type=QuicPacketType.INITIAL,
+                    sent_bytes=658,
+                ),
+                QuicSentPacket(
+                    epoch=Epoch.ONE_RTT,
+                    in_flight=True,
+                    is_ack_eliciting=True,
+                    is_crypto_packet=False,
+                    packet_number=1,
+                    packet_type=QuicPacketType.ZERO_RTT,
+                    sent_bytes=144,
                 ),
             ],
         )
@@ -150,21 +194,44 @@ class QuicPacketBuilderTest(TestCase):
         builder = create_builder()
         crypto = create_crypto()
 
-        # INITIAL
-        builder.start_packet(PACKET_TYPE_INITIAL, crypto)
+        # INITIAL with ACK + CRYPTO + PADDING
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
         self.assertEqual(builder.remaining_flight_space, 1156)
+
+        buf = builder.start_frame(QuicFrameType.ACK)
+        buf.push_bytes(bytes(16))
+
         buf = builder.start_frame(QuicFrameType.CRYPTO)
         buf.push_bytes(bytes(100))
         self.assertFalse(builder.packet_is_empty)
 
         # INITIAL, empty
-        builder.start_packet(PACKET_TYPE_INITIAL, crypto)
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
+        self.assertTrue(builder.packet_is_empty)
+
+        # HANDSHAKE with CRYPTO
+        builder.start_packet(QuicPacketType.HANDSHAKE, crypto)
+        self.assertEqual(builder.remaining_flight_space, 995)
+
+        buf = builder.start_frame(QuicFrameType.CRYPTO)
+        buf.push_bytes(bytes(994))
+        self.assertFalse(builder.packet_is_empty)
+
+        # HANDSHAKE with CRYPTO
+        builder.start_packet(QuicPacketType.HANDSHAKE, crypto)
+        self.assertEqual(builder.remaining_flight_space, 1157)
+
+        buf = builder.start_frame(QuicFrameType.CRYPTO)
+        buf.push_bytes(bytes(800))
+        self.assertFalse(builder.packet_is_empty)
+
+        # HANDSHAKE, empty
+        builder.start_packet(QuicPacketType.HANDSHAKE, crypto)
         self.assertTrue(builder.packet_is_empty)
 
         # check datagrams
         datagrams, packets = builder.flush()
-        self.assertEqual(len(datagrams), 1)
-        self.assertEqual(len(datagrams[0]), 1200)
+        self.assertEqual(datagram_sizes(datagrams), [1200, 844])
         self.assertEqual(
             packets,
             [
@@ -174,8 +241,66 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=True,
                     packet_number=0,
-                    packet_type=PACKET_TYPE_INITIAL,
-                    sent_bytes=1200,
+                    packet_type=QuicPacketType.INITIAL,
+                    sent_bytes=162,
+                ),
+                QuicSentPacket(
+                    epoch=Epoch.HANDSHAKE,
+                    in_flight=True,
+                    is_ack_eliciting=True,
+                    is_crypto_packet=True,
+                    packet_number=1,
+                    packet_type=QuicPacketType.HANDSHAKE,
+                    sent_bytes=1038,
+                ),
+                QuicSentPacket(
+                    epoch=Epoch.HANDSHAKE,
+                    in_flight=True,
+                    is_ack_eliciting=True,
+                    is_crypto_packet=True,
+                    packet_number=2,
+                    packet_type=QuicPacketType.HANDSHAKE,
+                    sent_bytes=844,
+                ),
+            ],
+        )
+
+        # check builder
+        self.assertEqual(builder.packet_number, 3)
+
+    def test_long_header_initial_server_without_handshake(self):
+        builder = create_builder()
+        crypto = create_crypto()
+
+        # INITIAL
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
+        self.assertEqual(builder.remaining_flight_space, 1156)
+        buf = builder.start_frame(QuicFrameType.CRYPTO)
+        buf.push_bytes(bytes(100))
+        self.assertFalse(builder.packet_is_empty)
+
+        # INITIAL, empty
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
+        self.assertTrue(builder.packet_is_empty)
+
+        # HANDSHAKE, empty
+        builder.start_packet(QuicPacketType.HANDSHAKE, crypto)
+        self.assertTrue(builder.packet_is_empty)
+
+        # check datagrams
+        datagrams, packets = builder.flush()
+        self.assertEqual(datagram_sizes(datagrams), [1200])
+        self.assertEqual(
+            packets,
+            [
+                QuicSentPacket(
+                    epoch=Epoch.INITIAL,
+                    in_flight=True,
+                    is_ack_eliciting=True,
+                    is_crypto_packet=True,
+                    packet_number=0,
+                    packet_type=QuicPacketType.INITIAL,
+                    sent_bytes=145,
                 )
             ],
         )
@@ -192,7 +317,7 @@ class QuicPacketBuilderTest(TestCase):
         crypto = create_crypto()
 
         # HANDSHAKE, with only a PING frame
-        builder.start_packet(PACKET_TYPE_HANDSHAKE, crypto)
+        builder.start_packet(QuicPacketType.HANDSHAKE, crypto)
         builder.start_frame(QuicFrameType.PING)
         self.assertFalse(builder.packet_is_empty)
 
@@ -209,7 +334,7 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=False,
                     packet_number=0,
-                    packet_type=PACKET_TYPE_HANDSHAKE,
+                    packet_type=QuicPacketType.HANDSHAKE,
                     sent_bytes=45,
                 )
             ],
@@ -220,25 +345,25 @@ class QuicPacketBuilderTest(TestCase):
         crypto = create_crypto()
 
         # INITIAL, full length
-        builder.start_packet(PACKET_TYPE_INITIAL, crypto)
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
         self.assertEqual(builder.remaining_flight_space, 1156)
         buf = builder.start_frame(QuicFrameType.CRYPTO)
         buf.push_bytes(bytes(builder.remaining_flight_space))
         self.assertFalse(builder.packet_is_empty)
 
         # INITIAL, empty
-        builder.start_packet(PACKET_TYPE_INITIAL, crypto)
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
         self.assertTrue(builder.packet_is_empty)
 
         # ONE_RTT, full length
-        builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+        builder.start_packet(QuicPacketType.ONE_RTT, crypto)
         self.assertEqual(builder.remaining_flight_space, 1173)
         buf = builder.start_frame(QuicFrameType.STREAM_BASE)
         buf.push_bytes(bytes(builder.remaining_flight_space))
         self.assertFalse(builder.packet_is_empty)
 
         # ONE_RTT, empty
-        builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+        builder.start_packet(QuicPacketType.ONE_RTT, crypto)
         self.assertTrue(builder.packet_is_empty)
 
         # check datagrams
@@ -255,7 +380,7 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=True,
                     packet_number=0,
-                    packet_type=PACKET_TYPE_INITIAL,
+                    packet_type=QuicPacketType.INITIAL,
                     sent_bytes=1200,
                 ),
                 QuicSentPacket(
@@ -264,7 +389,7 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=False,
                     packet_number=1,
-                    packet_type=PACKET_TYPE_ONE_RTT,
+                    packet_type=QuicPacketType.ONE_RTT,
                     sent_bytes=1200,
                 ),
             ],
@@ -273,37 +398,44 @@ class QuicPacketBuilderTest(TestCase):
         # check builder
         self.assertEqual(builder.packet_number, 2)
 
-    def test_long_header_then_long_header(self):
+    def test_long_header_then_long_header_then_short_header(self):
         builder = create_builder()
         crypto = create_crypto()
 
         # INITIAL
-        builder.start_packet(PACKET_TYPE_INITIAL, crypto)
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
         self.assertEqual(builder.remaining_flight_space, 1156)
         buf = builder.start_frame(QuicFrameType.CRYPTO)
         buf.push_bytes(bytes(199))
         self.assertFalse(builder.packet_is_empty)
 
+        # INITIAL, empty
+        builder.start_packet(QuicPacketType.INITIAL, crypto)
+        self.assertTrue(builder.packet_is_empty)
+
         # HANDSHAKE
-        builder.start_packet(PACKET_TYPE_HANDSHAKE, crypto)
-        self.assertEqual(builder.remaining_flight_space, 1157)
+        builder.start_packet(QuicPacketType.HANDSHAKE, crypto)
+        self.assertEqual(builder.remaining_flight_space, 913)
         buf = builder.start_frame(QuicFrameType.CRYPTO)
         buf.push_bytes(bytes(299))
         self.assertFalse(builder.packet_is_empty)
-        self.assertEqual(builder.remaining_flight_space, 857)
+        self.assertEqual(builder.remaining_flight_space, 613)
 
-        # ONE_RTT
-        builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
-        self.assertEqual(builder.remaining_flight_space, 830)
+        # HANDSHAKE, empty
+        builder.start_packet(QuicPacketType.HANDSHAKE, crypto)
+        self.assertTrue(builder.packet_is_empty)
+
+        # ONE_RTT, padded
+        builder.start_packet(QuicPacketType.ONE_RTT, crypto)
+        self.assertEqual(builder.remaining_flight_space, 586)
         buf = builder.start_frame(QuicFrameType.CRYPTO)
-        buf.push_bytes(bytes(299))
+        buf.push_bytes(bytes(300))
         self.assertFalse(builder.packet_is_empty)
 
         # check datagrams
         datagrams, packets = builder.flush()
-        self.assertEqual(len(datagrams), 2)
+        self.assertEqual(len(datagrams), 1)
         self.assertEqual(len(datagrams[0]), 1200)
-        self.assertEqual(len(datagrams[1]), 670)
         self.assertEqual(
             packets,
             [
@@ -313,8 +445,8 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=True,
                     packet_number=0,
-                    packet_type=PACKET_TYPE_INITIAL,
-                    sent_bytes=1200,
+                    packet_type=QuicPacketType.INITIAL,
+                    sent_bytes=244,
                 ),
                 QuicSentPacket(
                     epoch=Epoch.HANDSHAKE,
@@ -322,7 +454,7 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=True,
                     packet_number=1,
-                    packet_type=PACKET_TYPE_HANDSHAKE,
+                    packet_type=QuicPacketType.HANDSHAKE,
                     sent_bytes=343,
                 ),
                 QuicSentPacket(
@@ -331,8 +463,8 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=True,
                     packet_number=2,
-                    packet_type=PACKET_TYPE_ONE_RTT,
-                    sent_bytes=327,
+                    packet_type=QuicPacketType.ONE_RTT,
+                    sent_bytes=613,  # includes padding
                 ),
             ],
         )
@@ -344,7 +476,7 @@ class QuicPacketBuilderTest(TestCase):
         builder = create_builder()
         crypto = create_crypto()
 
-        builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+        builder.start_packet(QuicPacketType.ONE_RTT, crypto)
         self.assertEqual(builder.remaining_flight_space, 1173)
         self.assertTrue(builder.packet_is_empty)
 
@@ -356,12 +488,12 @@ class QuicPacketBuilderTest(TestCase):
         # check builder
         self.assertEqual(builder.packet_number, 0)
 
-    def test_short_header_padding(self):
+    def test_short_header_full_length(self):
         builder = create_builder()
         crypto = create_crypto()
 
         # ONE_RTT, full length
-        builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+        builder.start_packet(QuicPacketType.ONE_RTT, crypto)
         self.assertEqual(builder.remaining_flight_space, 1173)
         buf = builder.start_frame(QuicFrameType.CRYPTO)
         buf.push_bytes(bytes(builder.remaining_flight_space))
@@ -380,7 +512,7 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=True,
                     packet_number=0,
-                    packet_type=PACKET_TYPE_ONE_RTT,
+                    packet_type=QuicPacketType.ONE_RTT,
                     sent_bytes=1200,
                 )
             ],
@@ -398,14 +530,14 @@ class QuicPacketBuilderTest(TestCase):
 
         crypto = create_crypto()
 
-        builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+        builder.start_packet(QuicPacketType.ONE_RTT, crypto)
         self.assertEqual(builder.remaining_flight_space, 973)
         buf = builder.start_frame(QuicFrameType.CRYPTO)
         buf.push_bytes(bytes(builder.remaining_flight_space))
         self.assertFalse(builder.packet_is_empty)
 
         with self.assertRaises(QuicPacketBuilderStop):
-            builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+            builder.start_packet(QuicPacketType.ONE_RTT, crypto)
             builder.start_frame(QuicFrameType.CRYPTO)
 
         # check datagrams
@@ -421,7 +553,7 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=True,
                     packet_number=0,
-                    packet_type=PACKET_TYPE_ONE_RTT,
+                    packet_type=QuicPacketType.ONE_RTT,
                     sent_bytes=1000,
                 ),
             ],
@@ -442,7 +574,7 @@ class QuicPacketBuilderTest(TestCase):
         crypto = create_crypto()
 
         with self.assertRaises(QuicPacketBuilderStop):
-            builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+            builder.start_packet(QuicPacketType.ONE_RTT, crypto)
             builder.start_frame(QuicFrameType.CRYPTO)
 
         # check datagrams
@@ -463,12 +595,12 @@ class QuicPacketBuilderTest(TestCase):
 
         crypto = create_crypto()
 
-        builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+        builder.start_packet(QuicPacketType.ONE_RTT, crypto)
         buf = builder.start_frame(QuicFrameType.ACK)
         buf.push_bytes(bytes(64))
 
         with self.assertRaises(QuicPacketBuilderStop):
-            builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+            builder.start_packet(QuicPacketType.ONE_RTT, crypto)
             builder.start_frame(QuicFrameType.CRYPTO)
 
         # check datagrams
@@ -484,7 +616,7 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=False,
                     is_crypto_packet=False,
                     packet_number=0,
-                    packet_type=PACKET_TYPE_ONE_RTT,
+                    packet_type=QuicPacketType.ONE_RTT,
                     sent_bytes=92,
                 ),
             ],
@@ -503,7 +635,7 @@ class QuicPacketBuilderTest(TestCase):
         crypto = create_crypto()
 
         with self.assertRaises(QuicPacketBuilderStop):
-            builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+            builder.start_packet(QuicPacketType.ONE_RTT, crypto)
 
         # check datagrams
         datagrams, packets = builder.flush()
@@ -522,14 +654,14 @@ class QuicPacketBuilderTest(TestCase):
 
         crypto = create_crypto()
 
-        builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+        builder.start_packet(QuicPacketType.ONE_RTT, crypto)
         self.assertEqual(builder.remaining_flight_space, 773)
         buf = builder.start_frame(QuicFrameType.CRYPTO)
         buf.push_bytes(bytes(builder.remaining_flight_space))
         self.assertFalse(builder.packet_is_empty)
 
         with self.assertRaises(QuicPacketBuilderStop):
-            builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+            builder.start_packet(QuicPacketType.ONE_RTT, crypto)
 
         # check datagrams
         datagrams, packets = builder.flush()
@@ -544,7 +676,7 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=True,
                     packet_number=0,
-                    packet_type=PACKET_TYPE_ONE_RTT,
+                    packet_type=QuicPacketType.ONE_RTT,
                     sent_bytes=800,
                 )
             ],
@@ -559,20 +691,20 @@ class QuicPacketBuilderTest(TestCase):
 
         crypto = create_crypto()
 
-        builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+        builder.start_packet(QuicPacketType.ONE_RTT, crypto)
         self.assertEqual(builder.remaining_flight_space, 1173)
         buf = builder.start_frame(QuicFrameType.CRYPTO)
         buf.push_bytes(bytes(builder.remaining_flight_space))
         self.assertFalse(builder.packet_is_empty)
 
-        builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+        builder.start_packet(QuicPacketType.ONE_RTT, crypto)
         self.assertEqual(builder.remaining_flight_space, 773)
         buf = builder.start_frame(QuicFrameType.CRYPTO)
         buf.push_bytes(bytes(builder.remaining_flight_space))
         self.assertFalse(builder.packet_is_empty)
 
         with self.assertRaises(QuicPacketBuilderStop):
-            builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+            builder.start_packet(QuicPacketType.ONE_RTT, crypto)
 
         # check datagrams
         datagrams, packets = builder.flush()
@@ -588,7 +720,7 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=True,
                     packet_number=0,
-                    packet_type=PACKET_TYPE_ONE_RTT,
+                    packet_type=QuicPacketType.ONE_RTT,
                     sent_bytes=1200,
                 ),
                 QuicSentPacket(
@@ -597,7 +729,7 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=True,
                     packet_number=1,
-                    packet_type=PACKET_TYPE_ONE_RTT,
+                    packet_type=QuicPacketType.ONE_RTT,
                     sent_bytes=800,
                 ),
             ],
@@ -615,7 +747,7 @@ class QuicPacketBuilderTest(TestCase):
         crypto = create_crypto()
 
         # HANDSHAKE, with only a PING frame
-        builder.start_packet(PACKET_TYPE_ONE_RTT, crypto)
+        builder.start_packet(QuicPacketType.ONE_RTT, crypto)
         builder.start_frame(QuicFrameType.PING)
         self.assertFalse(builder.packet_is_empty)
 
@@ -632,7 +764,7 @@ class QuicPacketBuilderTest(TestCase):
                     is_ack_eliciting=True,
                     is_crypto_packet=False,
                     packet_number=0,
-                    packet_type=PACKET_TYPE_ONE_RTT,
+                    packet_type=QuicPacketType.ONE_RTT,
                     sent_bytes=29,
                 )
             ],
